/*
Each resolver contains 4 args:
1. parent: The object contains the result returned from the resolver on the parent field on in case of a top-level Query field, the rootValue passed from the serverConfiguration
2. args: an object  with the args passed into the field in the query
3. context : object shared by all resolversis used to contain per request state
4. info: This argument contains information about the execution state of the query, including the field name, path to the field from the root, and more.
*/

const { ApolloError } = require('apollo-server');
const { GraphQLScalarType } = require('graphql');
const { Kind } = require('graphql/language');
const joi = require('joi');
const { sortBy } = require('lodash');
const {
    getSupportedVolumes, setupTiering, adjustClusterResponse, discoverClusterOperation, addToTier,
    getTieredVolumesSummary, validateTiering
} = require('../operations/cloud-manager-operations');
const authenticate = require('../operations/authentication-operations');
const adjustAgentResponse = require('../operations/agents-operations');
const { findWorkspace, findAccount } = require('../operations/multi-tenancy-operation');
const { getAgents, getAgent } = require('../lib/cloud-manager/agents');
const { getAllClustersInWorkspace } = require('../operations/multi-tenancy-operation');
const { CLOUD_TIERING_NAMESPACE, AZURE, AWS } = require('../utils/consts');
const getCallerIdentity = require('../lib/aws/sts');
const getLogger = require('../utils/logger/logger');
const getRegions = require('../operations/regions-operations');

const logger = getLogger();

const authorizeAndFindAccount = next => async (root, args, context, info) => {
    const { authorizationToken } = context;
    // authenticate
    await authenticate(authorizationToken);
    // find account
    const accountId = await findAccount(CLOUD_TIERING_NAMESPACE.get('userPortalId'));
    CLOUD_TIERING_NAMESPACE.set('accountId', accountId);

    return next(root, args, context, info);
};

const resolvers = {
    Query: {
        agents: authorizeAndFindAccount(async (parent, args) => {
            const { agentId } = args;
            const accountId = CLOUD_TIERING_NAMESPACE.get('accountId');
            if (agentId) {
                const agent = await getAgent(agentId, accountId);
                logger.info(`received agent ${agentId} of accountId ${accountId}`);
                return { agents: [adjustAgentResponse(agent)], accountId };
            }
            const { agents } = await getAgents(accountId);
            logger.info(`received agents of accountId ${accountId} ${JSON.stringify(agents)}`);
            return { agents: agents.map(agent => adjustAgentResponse(agent)), accountId };
        }),
        clusters: authorizeAndFindAccount(async (parent, args) => {
            const { clusterId } = args;
            const accountId = CLOUD_TIERING_NAMESPACE.get('accountId');
            // check for account and workspaces if there are no accounts and workspaces
            const workspaceId = await findWorkspace(accountId);
            const clusters = await getAllClustersInWorkspace(accountId, workspaceId, clusterId);
            logger.info(`Received clusters ${JSON.stringify(clusters)} trying to adjust`);
            if (clusterId && clusters.length === 0) {
                throw new ApolloError(`Cluster with id ${clusterId} was not found`, 404);
            }
            const adjustedClusters = await Promise.all(clusters.map(cluster => adjustClusterResponse(cluster)));
            return sortBy(adjustedClusters, 'name');
        }),
        volumes: authorizeAndFindAccount(async (parent, args) => {
            const { clusterId } = args;
            logger.info(`Get volumes request for cluster ${clusterId}`);
            return getSupportedVolumes(clusterId);
        }),
        regions: authorizeAndFindAccount(async (parent, args) => {
            const {
                type, regionCode, clusterId
            } = args;
            return getRegions(type, regionCode, clusterId);
        })
    },
    Mutation: {
        discover: authorizeAndFindAccount(async (parent, args) => {
            const accountId = CLOUD_TIERING_NAMESPACE.get('accountId');
            const {
                input: {
                    clusterAddress, clusterUserName, clusterPassword, agentId
                }
            } = args;
            const workspaceId = await findWorkspace(accountId);
            const clusterId = await discoverClusterOperation(clusterAddress, clusterUserName, clusterPassword, workspaceId, agentId, accountId);
            return { clusterId };
        }),
        startTiering: authorizeAndFindAccount(async (parent, args) => {
            const {
                input: {
                    clusterId, volumeIdentifiers, providerProperties: {
                        cloudProvider, awsKeys, region, storageClass
                    }, tieringPolicy
                }
            } = args;

            if (![AZURE, AWS].includes(cloudProvider)) {
                throw new ApolloError(`provider ${cloudProvider} is not supported`, 403);
            }
            if (cloudProvider === AWS) {
                if (!awsKeys) {
                    throw new ApolloError(`for tiering to ${cloudProvider} you must provide aws keys`, 400);
                }
                const { accessKey, secretKey } = awsKeys;
                // check validity of access keys
                await getCallerIdentity(accessKey, secretKey);
            }
            // validate deails -
            await validateTiering(clusterId, tieringPolicy);

            // no await because ngnix closes the connection after 70 seconds and it might take longer
            setupTiering(cloudProvider, clusterId, region, awsKeys, volumeIdentifiers, storageClass, tieringPolicy);
            return { status: 'success' };
        }),
        addToTier: authorizeAndFindAccount(async (parent, args) => {
            const {
                input: {
                    clusterId, volumeIdentifiers, tieringPolicy
                }
            } = args;
            await validateTiering(clusterId, tieringPolicy);
            addToTier(clusterId, volumeIdentifiers, tieringPolicy);
            return { status: 'success' };
        })
    },
    Cluster: {
        volumes: async parent => {
            if (parent.status.status === 'ON') {
                return getTieredVolumesSummary(parent.clusterId);
            }
        }
    },
    IpAddress: new GraphQLScalarType({
        name: 'IpAddress',
        description: 'Ip Address custom scalar type',
        parseValue(value) { // value from the client (ui not from playground)
            const { error } = joi.validate(value, joi.string().ip({ versions: ['ipv4'] }).error(new Error('Invalid Ip Address'))); // TypeError
            if (error) {
                throw ApolloError(error.message, 400);
            }
            return value;
        },
        serialize(value) { // value sent to the client
            const { error } = joi.validate(value, joi.string().ip({ versions: ['ipv4'] }).error(new Error('Invalid Ip Address'))); // TypeError
            if (error) {
                throw ApolloError(error.message);
            }
            return value;
        },
        parseLiteral(ast) {
            if (ast.kind === Kind.STRING) {
                const { error } = joi.validate(ast.value, joi.string().ip({ versions: ['ipv4'] }).error(new Error('Invalid Ip Address'))); // TypeError
                if (error) {
                    throw ApolloError(error.message, 400);
                }
                return ast.value;
            }
            return null;
        }
    })
};

module.exports = resolvers;
