const AWS = require('aws-sdk');
const { compact, flatten } = require('lodash');
const { SERVICE_REGION } = require('../../utils/consts');
const getLogger = require('../../utils/logger/logger');

const logger = getLogger();

function getSQS() {
    return new AWS.SQS({ region: SERVICE_REGION });
}

async function createQueue(name) {
    const sqsClient = getSQS();
    const data = await sqsClient.createQueue({
        QueueName: name,
        Attributes: {
            ReceiveMessageWaitTimeSeconds: '20',
            VisibilityTimeout: '60'
        }
    }).promise();
    return data.QueueUrl;
}

async function deleteMessage(queueUrl, receiptHandle) {
    const sqsClient = getSQS();
    return sqsClient.deleteMessage({
        QueueUrl: queueUrl,
        ReceiptHandle: receiptHandle
    }).promise();
}


function parseMessages(messages, queueUrl) {
    const parsedMessages = messages.map(message => {
        let parsedMessage;
        try {
            parsedMessage = {
                body: JSON.parse(message.Body),
                receiptHandle: message.ReceiptHandle
            };
        } catch (err) {
            logger.warn('Failed to parse message body', message.Body);
            deleteMessage(queueUrl, message.ReceiptHandle);
        }
        return parsedMessage;
    });
    return compact(flatten(parsedMessages));
}

async function receiveMessage(queueUrl) {
    const sqsClient = getSQS();
    const data = await sqsClient.receiveMessage({
        QueueUrl: queueUrl,
        MaxNumberOfMessages: 10,
        WaitTimeSeconds: 20
    }).promise();
    return parseMessages(data.Messages || [], queueUrl); // if there are no messages there is Messages field and the function fails
}

async function getQueueArn(queueUrl) {
    const sqsClient = getSQS();
    const data = await sqsClient.getQueueAttributes({
        QueueUrl: queueUrl,
        AttributeNames: ['QueueArn']
    }).promise();
    return data.Attributes.QueueArn;
}

async function setQueueAttribute(queueUrl, attributesObject) {
    const sqsClient = getSQS();
    return sqsClient.setQueueAttributes({
        QueueUrl: queueUrl,
        Attributes: attributesObject
    }).promise();
}

async function getQueueAttributes(queueUrl, attributeNames) {
    const sqsClient = getSQS();
    const response = await sqsClient.getQueueAttributes({ QueueUrl: queueUrl, AttributeNames: attributeNames });
    return response.Attributes;
}

async function allowArnSendMessage(queueUrl, queueArn, arn) {
    const policy = {
        Version: '2012-10-17',
        Id: 'arn-send-message',
        Statement: [
            {
                Sid: `Sid${Date.now()}`,
                Effect: 'Allow',
                Principal: '*',
                Action: 'SQS:SendMessage',
                Resource: queueArn,
                Condition: {
                    ArnEquals: {
                        'aws:SourceArn': arn
                    }
                }
            }
        ]
    };

    return setQueueAttribute(queueUrl, {
        Policy: JSON.stringify(policy)
    });
}

module.exports = {
    getQueueAttributes, getQueueArn, deleteMessage, receiveMessage, createQueue, allowArnSendMessage
};
