const http = require('http');
const ms = require('ms');
const config = require('config');
const { get } = require('lodash');
const axios = require('axios');
const axiosRetry = require('axios-retry');
const { hideToken } = require('../../utils/utils');
const HttpError = require('../../utils/errors');
const getLogger = require('../../utils/logger/logger');
const { CLOUD_TIERING_NAMESPACE } = require('../../utils/consts');

const logger = getLogger('axios');

axiosRetry(axios, { retries: config.get('retries') });

axios.default.interceptors.request.use(request => {
    logger.info(`AXIOS request: ${request.method.toUpperCase()} ${request.url}`, request.data ? JSON.stringify(hideToken(request.data)) : '');
    return request;
});

axios.default.interceptors.response.use(response => {
    logger.info(`AXIOS response: ${response.status} (${response.statusText}) - ${response.config.method.toUpperCase()} ${response.config.url}`, JSON.stringify(hideToken(response.data)));
    return response;
}, error => {
    logger.info(`AXIOS error :status ${error.code} ${error.response ? error.response.status : error.response} error : ${error.response ? JSON.stringify(error.response.data) : error.Error}`);
    return Promise.reject(error);
});

function extractError(error) {
    if (error.response) {
        // HTTP response was received
        throw new HttpError(get(error, 'response.data.message', error.response.data), error.response.status);
    } else {
        // client side error - connection refused, aborted, timeout, etc
        throw new HttpError(error.Error, 500);
    }
}

function createAxiosConfig(headersToAdd) {
    const headers = {
        ...{
            authorization: CLOUD_TIERING_NAMESPACE.get('userToken'),
            'X-Service-Request-Id': CLOUD_TIERING_NAMESPACE.get('requestId')
        },
        ...headersToAdd
    };
    return {
        headers,
        timeout: ms(config.get('timeout')),
        httpAgent: new http.Agent({ keepAlive: true })
    };
}

async function postRequest(url, body, headers) {
    try {
        const { data } = await axios.post(url, body, createAxiosConfig(headers));
        return data;
    } catch (err) {
        extractError(err);
    }
}

async function getRequest(url, headers) {
    try {
        const { data } = await axios.get(url, createAxiosConfig(headers));
        return data;
    } catch (err) {
        extractError(err);
    }
}

async function putRequest(url, body, headers) {
    try {
        const { data } = await axios.put(url, body, createAxiosConfig(headers));
        return data;
    } catch (err) {
        extractError(err);
    }
}

async function deleteRequest(url) {
    try {
        const { data } = await axios.delete(url, createAxiosConfig());
        return data;
    } catch (err) {
        extractError(err);
    }
}


module.exports = {
    postRequest, getRequest, putRequest, deleteRequest
};
