const {
    getAccountsSchema, getWorkspaceSchema, createAccountResponseSchema, workspaceSchema, resourcesSchema
} = require('../../joi-schemas/multi-tenancy-schemas');
const { validateResponse } = require('../../utils/schema-validation');
const {
    MULTI_TENANCY_ENDPOINT, API_PATH_ACCOUNTS, DISCOVER_LOCATION, API_PATH_RESOURCES, API_PATH_RESOURCES_INTERNAL, CLOUD_TIERING_NAMESPACE
} = require('../../utils/consts');
const getLogger = require('../../utils/logger/logger');
const { getTenancyServiceToken } = require('../auth0');
const { getRequest, postRequest } = require('./cloud-manager-axios-utils');

const logger = getLogger();


async function getAccounts() {
    logger.info('get Accounts');
    const headers = {
        'X-User-Token': CLOUD_TIERING_NAMESPACE.get('userToken'),
        authorization: getTenancyServiceToken()
    };
    const data = await getRequest(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}`, headers);
    validateResponse(data, getAccountsSchema);
    return data;
}

async function createAccount(accountName) {
    logger.info(`create account ${accountName}`);
    const data = await postRequest(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountName}`, undefined, {
        'X-User-Token': CLOUD_TIERING_NAMESPACE.get('userToken'),
        authorization: getTenancyServiceToken()
    });
    validateResponse(data, createAccountResponseSchema);
    return data;
}

async function getWorkspaces(accountId) {
    logger.info(`get workspaces of account ${accountId}`);
    const data = await getRequest(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountId}/workspace`, {
        'X-User-Token': CLOUD_TIERING_NAMESPACE.get('userToken'),
        authorization: getTenancyServiceToken()
    });
    validateResponse(data, getWorkspaceSchema);
    return data;
}

async function getResourcesByWorkspace(accountId, workspaceId) {
    logger.info(`get resources by workspace- workspace ${workspaceId} and account ${accountId}`);
    const data = await getRequest(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES}?workspace=${workspaceId}&account=${accountId}&resource-type=${DISCOVER_LOCATION}`,
        {
            'X-User-Token': CLOUD_TIERING_NAMESPACE.get('userToken'),
            authorization: getTenancyServiceToken()
        });
    validateResponse(data, resourcesSchema);
    return data;
}

async function getResourcesByAccount(accountId) {
    logger.info(`get resources by  account ${accountId}`);
    const data = await getRequest(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES}?account=${accountId}&resource-type=${DISCOVER_LOCATION}`,
        {
            'X-User-Token': CLOUD_TIERING_NAMESPACE.get('userToken'),
            authorization: getTenancyServiceToken()
        });
    validateResponse(data, resourcesSchema);
    return data;
}

async function getResourcesByAccountInternal(accountId) {
    logger.info(`get resources by  account ${accountId}`);
    const data = await getRequest(`${MULTI_TENANCY_ENDPOINT}${API_PATH_RESOURCES_INTERNAL}?account=${accountId}&resource-type=${DISCOVER_LOCATION}`, {
        'X-User-Token': CLOUD_TIERING_NAMESPACE.get('userToken'),
        authorization: getTenancyServiceToken()
    });
    validateResponse(data, resourcesSchema);
    return data;
}


async function createWorkspace(accountId, workspaceName) {
    logger.info(`create workspace with name ${workspaceName} for account ${accountId}`);
    const data = await postRequest(`${MULTI_TENANCY_ENDPOINT}${API_PATH_ACCOUNTS}/${accountId}/workspace/${workspaceName}`, undefined, {
        'X-User-Token': CLOUD_TIERING_NAMESPACE.get('userToken'),
        authorization: getTenancyServiceToken()
    });
    validateResponse(data, workspaceSchema);
    return data;
}


module.exports = {
    getAccounts,
    getWorkspaces,
    createWorkspace,
    createAccount,
    getResourcesByWorkspace,
    getResourcesByAccount,
    getResourcesByAccountInternal
};
