const bytes = require('bytes');
// const config = require('config');
const { flatten, sum } = require('lodash');
const Account = require('../models/account');
const Metering = require('../models/metering');
const getLogger = require('../utils/logger/logger');
const { getAllClustersInAccountInternal } = require('../operations/multi-tenancy-operation');
const { getVolumes, modifyVolume } = require('../lib/cloud-manager/volumes');
const { SAAS_REGISTRATION_STATUS_SUBSCRIBE_SUCCESS /* AGENDA_JOB_NAME_SAAS_METERING */ } = require('../utils/consts');
const { batchMeterUsage } = require('../lib/aws/marketplace-metering');

const logger = getLogger();


// #1 find all bilable accounts
// #2 for each account getworkspace
// #3 for each workspace clustersInfo
// #4 sum all the usedCapacity( or other field )
// #5 create metering entry
// #6 send to AWS
// #7 insert to the saasBilling collectionand send again the failed ones


async function createAccountAndClusterObject(accountId) {
    const clusters = await getAllClustersInAccountInternal(accountId);
    return { accountId, clusters };
}

async function getUsedCapacityOfAccount(serviceToken, clusters) {
    const usedCapacityArray = clusters.map(cluster => {
        if (cluster.capacityFeatures && cluster.capacityFeatures.objectStoreConfig) {
            const { capacityFeatures: { objectStoreConfig: { usedSpace: { size, unit } } } } = cluster;
            return bytes(`${size}${unit}`);
        }
        return 0;
    });
    return sum(usedCapacityArray);
}


async function getMeteringInfoForAccount(serviceToken, account) {
    const accountWithClusters = await createAccountAndClusterObject(serviceToken, account.accountId);
    const usedCapacity = await getUsedCapacityOfAccount(serviceToken, accountWithClusters.clusters);
    return { customerIdentifier: account.awsSubscription.customerIdentifier, usedCapacity };
}

// bill all accounts that their subscirption status is not unsubscribe-suceess and trial expired
async function billableAccounts() {
    return Account.find({
        $and: [{ 'awsSubscription.status': { $eq: SAAS_REGISTRATION_STATUS_SUBSCRIBE_SUCCESS } },
            { trialExpiry: { $exists: true, $lt: Date.now() }, 'awsSubscription.customerIdentifier': { $exists: true } }]
    }).exec();
}


async function sendRecordsToAWS(records) {
    const responses = await batchMeterUsage(records);
    const processedRecords = flatten(responses.map(response => response.Results));
    logger.info('processed billing entries ', processedRecords);
    await Metering.insertMany(processedRecords);
    return flatten(responses.map(response => response.UnprocessedRecords));
}

async function billAccounts(serviceToken, chargingTime) {
    // #1 find all billable accounts
    const accounts = await billableAccounts();
    // #2 for each account find its usedCapacity [{customerIdentifier, usedCapacity}]
    const accountsAndUsedCapacity = await Promise.all(accounts.map(account => getMeteringInfoForAccount(serviceToken, account)));
    // #3 create metering record for each account
    const meteringRecords = accountsAndUsedCapacity.map(accountObject => {
        const { customerIdentifier, quantity } = accountObject;
        return {
            CustomerIdentifier: customerIdentifier,
            Quantity: quantity,
            Timestamp: chargingTime
        };
    });
    // #4 send batch metering ( twice )
    const unprocessedRecordsFirstTry = await sendRecordsToAWS(meteringRecords);
    const unprocessedRecordsSecondTry = await sendRecordsToAWS(unprocessedRecordsFirstTry);
    await Metering.insertMany(unprocessedRecordsSecondTry.map(record => ({
        Status: 'Failed',
        UsageRecord: record
    })));
}

async function getAllVolumesOfAccount(serviceToken, workSpaceObject) {
    const { clustersIds } = workSpaceObject;
    return Promise.all(clustersIds.map(async clusterId => {
        const volumes = await getVolumes(serviceToken, clusterId);
        return { clusterId, volumes };
    }));
}

async function stopTiering(serviceToken, volumeAndCluster) {
    const { clusterId, volumes } = volumeAndCluster;
    const promises = volumes.map(async volume => {
        const { svmName, name, tieringPolicy } = volume;
        if (tieringPolicy === 'auto') {
            await modifyVolume(serviceToken, clusterId, svmName, name, 'none');
        }
    });
    await Promise.all(promises);
}

async function stopTieringVolumesInAccount(serviceToken, accountId) {
    logger.info(`handeling unsubscribe of volumes in account ${accountId}`);
    // #2 find all clusters - {accountId, [clustersId]}
    const accountsWithClusters = await createAccountAndClusterObject(serviceToken, accountId);
    // #3 find all volumes of workspace = [{clusterId, volumes}]
    const clusterWithVolumes = await Promise.all(accountsWithClusters.map(accountObject => getAllVolumesOfAccount(serviceToken, accountObject)));
    // #4 change tier policy
    await Promise.all(clusterWithVolumes.map(volumeObject => stopTiering(serviceToken, volumeObject)));
}


module.exports = {
    getUsedCapacityOfAccount,
    createAccountAndClusterObject,
    getMeteringInfoForAccount,
    billableAccounts,
    billAccounts,
    sendRecordsToAWS,
    stopTieringVolumesInAccount
};
