const os = require('os');
const getLogger = require('./logger/logger');
const {
    MP_PRODUCT_CODE_METERING, MP_SUBSCRIPTION_STATUS_SNS_TOPIC, SELLER_ACCESS_KEY, SELLER_SECRET_KEY, SAAS_REGISTRATION_STATUS_UNSUBSCRIBE_SUCCESS
} = require('../utils/consts');
const { stopTieringVolumesInAccount } = require('../operations/marketplace-metering');
const Account = require('../models/account');

const {
    createQueue,
    receiveMessage,
    deleteMessage,
    getQueueArn,
    allowArnSendMessage
} = require('../lib/aws/sqs');
const { subscribe, confirmSubscription } = require('../lib/aws/sns');

const logger = getLogger();

function validateProductCode(code) {
    if (code !== MP_PRODUCT_CODE_METERING) {
        throw new Error(`Resolved product code is incorrect. Expected: ${MP_PRODUCT_CODE_METERING}, Got: ${code}`);
    }
}

async function handleMessage(serviceToken, message) {
    logger.info('handling message', JSON.stringify(message));
    const subscriptionMessage = JSON.parse(message.Message);
    const timestamp = new Date(message.Timestamp);
    const customerIdentifier = subscriptionMessage['customer-identifier'];
    validateProductCode(subscriptionMessage['product-code']);

    const account = await Account.findOne({
        'subscription.customerIdentifier': customerIdentifier
    }).exec();
    if (account) {
        if (timestamp > account.subscription.updatedAt) {
            const { accountId } = account;
            const { action } = subscriptionMessage;
            if (action === SAAS_REGISTRATION_STATUS_UNSUBSCRIBE_SUCCESS) {
                await stopTieringVolumesInAccount(serviceToken, accountId);
            }
            logger.info(`Updating Subscription status for account ${account.accountId}, status ${subscriptionMessage.action}, timestamp ${timestamp}`);
            account.subscription.status = subscriptionMessage.action;
            await account.save();
        } else {
            logger.error(`Received outdated subscription notification [Action: ${subscriptionMessage.action}. CustomerIdentifier: ${customerIdentifier}, timestamp ${timestamp}]`);
        }
    } else {
        logger.error(`Subscription notification's customer identifier not found. [CustomerIdentifier ${customerIdentifier}, Action ${subscriptionMessage.action}, timestamp ${timestamp}`);
    }
}


async function waitForSubscriptionConfirmation(queueUrl, count) {
    logger.info('Waiting for sns subscription confirmation ', count);
    const messages = await receiveMessage(queueUrl);
    const subscriptionConfirmationMessage = messages.find(message => message.body.Type && message.body.Type === 'SubscriptionConfirmation');
    if (subscriptionConfirmationMessage) {
        logger.info(`Got topic ${MP_SUBSCRIPTION_STATUS_SNS_TOPIC} subscription confirmation - confirming`);
        await confirmSubscription(MP_SUBSCRIPTION_STATUS_SNS_TOPIC, subscriptionConfirmationMessage.body.Token, SELLER_ACCESS_KEY, SELLER_SECRET_KEY);
        await deleteMessage(queueUrl, subscriptionConfirmationMessage.receiptHandle);
    } else if (count === 0) {
        throw new Error(`Queue ${queueUrl} didnt receive any subscription confirmation message`);
    } else {
        return waitForSubscriptionConfirmation(queueUrl, count - 1);
    }
}


async function pollQueue(serviceToken, queueUrl) {
    try {
        logger.debug(`Polling for messages from ${queueUrl}`);
        const messages = await receiveMessage(queueUrl);
        await Promise.all(messages.map(async message => {
            try {
                logger.debug('Got SQS message:', message);
                await handleMessage(serviceToken, message.body);
            } catch (err) {
                logger.error(`FAILED to handle message, ${JSON.stringify(message.body)}, with error ${JSON.stringify(err)}`);
            } finally {
                logger.debug(`Deleting message ${message} , receipt handle: ${message.receiptHandle}`);
                await deleteMessage(queueUrl, message.receiptHandle);
            }
        }));
        setImmediate(() => {
            pollQueue(queueUrl);
        });
    } catch (err) {
        logger.error(`Failed to poll messages from ${queueUrl}`, err);
        if (err.code === 'AWS.SimpleQueueService.NonExistentQueue') {
            logger.warn(`${queueUrl} does not exist - stopping polling`);
        } else {
            setTimeout(() => {
                pollQueue(queueUrl);
            }, 5000);
        }
    }
}

async function startMPSubscriptionStatusListener(serviceToken) {
    const prefix = 'CLOUD_TIERING_MP_SUBSCRIPTION_STATUS_';
    const name = `${prefix}${os.hostname().replace(/[-.]/g, '_')}`;
    // get queue url if returns undefined:
    logger.info(`Creating SQS queue ${name}`);
    const queueUrl = await createQueue(name);
    logger.info(`Queue ${queueUrl} successfully created`);
    const queueArn = await getQueueArn(queueUrl);
    logger.info(`Allow send message to queue ${queueArn} from topic ${MP_SUBSCRIPTION_STATUS_SNS_TOPIC}`);
    await allowArnSendMessage(queueUrl, queueArn, MP_SUBSCRIPTION_STATUS_SNS_TOPIC);
    logger.info(`Subscribing queue ${queueArn} to topic ${MP_SUBSCRIPTION_STATUS_SNS_TOPIC}`);
    const subscriptionArn = await subscribe(MP_SUBSCRIPTION_STATUS_SNS_TOPIC, queueArn);
    logger.info(`SQS ${queueArn} was successfully subscribed to SNS ${MP_SUBSCRIPTION_STATUS_SNS_TOPIC}, subscription ARN ${subscriptionArn}`);
    await waitForSubscriptionConfirmation(queueUrl, 5); // wait 120 seconds
    pollQueue(serviceToken, queueUrl);
}

module.exports = startMPSubscriptionStatusListener;
