const { validateResponse, validateRequest } = require('../../src/utils/schema-validation');
const {
    clusterMetadataResponseSchema, aggregatesResponseSchema, volumesResponseSchema, licenseStatusResponseSchema, discoverRequestSchema
} = require('../../src/joi-schemas/cloud-manager-schamas');
const { DISCOVER_LOCATION } = require('../../src/utils/consts');

function createCapacityObj(size = 1, unit = 'GB') {
    return {
        size,
        unit
    };
}

const clusterMetadataValid = {
    clusterName: 'sagiAFF',
    nodeModel: 'AFF1',
    ontapVersion: '9.5'
};

const discoverRequest = {
    tenantId: 'tenant-1', name: 'AFF-1', clusterAddress: '1.1.1.1', clusterUserName: 'Sagi', clusterPassword: 'Netapp123', location: DISCOVER_LOCATION
};

const discoverRequestInvalidAddress = {
    tenantId: 'tenant-1', name: 'AFF-1', clusterAddress: 'abcdefg', clusterUserName: 'Sagi', clusterPassword: 'Netapp123', location: 'ON_PREM'
};

const clusterMetadataInvalid = {
    serialNumber: '123456',
    clusterUuid: 'aff-123'
};

const aggregatesValid = [{
    name: 'aggr1',
    state: 'online',
    isObjectStoreAttachEligible: true,
    objectStoreName: 'object-store-1',
    isInactiveDataReportingEnabled: false,
    usedCapacity: {
        size: 0,
        unit: 'TB'
    }
}];

const emptyVolumesResponse = [];

const volumesValid = [{
    name: 'vol',
    svmName: 'svm_onprem',
    size: createCapacityObj(10),
    usedSize: createCapacityObj(5),
    tierInactiveUserData: createCapacityObj(2),
    tierInactiveUserDataPercent: 40,
    tieringPolicy: 'auto',
    capacityTierUsedSize: createCapacityObj(10),
    aggregateName: 'aggr1'
}];

const volumesPartialData = [{
    name: 'vol',
    svmName: 'svm_onprem',
    size: createCapacityObj(10),
    usedSize: createCapacityObj(5),
    tierInactiveUserData: null,
    tierInactiveUserDataPercent: null,
    tieringPolicy: 'auto',
    capacityTierUsedSize: createCapacityObj(10),
    aggregateName: 'aggr1'
}];

const volumesNoTieringPolicy = [{
    name: 'vol',
    svmName: 'svm_onprem',
    size: createCapacityObj(10),
    usedSize: createCapacityObj(5),
    tierInactiveUserData: null,
    tierInactiveUserDataPercent: null,
    tieringPolicy: null,
    capacityTierUsedSize: createCapacityObj(10),
    aggregateName: 'aggr1'
}];

const licenseStatusValid = {
    serialNumber: '123',
    type: 'capacity',
    expiration: '6/30/2020',
    detailedStatus: 'The system is using 2.39TB. The system can use up to 3TB.'
};

const licenseStatusNoExpiration = {
    serialNumber: '123',
    type: 'capacity',
    expiration: null,
    detailedStatus: 'The system is using 2.39TB. The system can use up to 3TB.'
};

describe('validate cloud-manager response schemas', () => {
    test('successfully validate cluster metadata response', () => {
        expect(validateResponse(clusterMetadataValid, clusterMetadataResponseSchema)).not.toBeDefined();
    });

    test('failed to validate invalid cluster metadata response', () => {
        function validateInvalidResponse() {
            return validateResponse(clusterMetadataInvalid, clusterMetadataResponseSchema);
        }
        expect(validateInvalidResponse).toThrow('child "clusterName" fails because ["clusterName" is required]');
    });

    test('successfully validate aggregates response', () => {
        expect(validateResponse(aggregatesValid, aggregatesResponseSchema)).not.toBeDefined();
    });

    test('successfully validate empty volumes response', () => {
        expect(validateResponse(emptyVolumesResponse, volumesResponseSchema)).not.toBeDefined();
    });

    test('successfully validate volumes response', () => {
        expect(validateResponse(volumesValid, volumesResponseSchema)).not.toBeDefined();
    });

    test('successfully validate volumes with partial data', () => {
        expect(validateResponse(volumesPartialData, volumesResponseSchema)).not.toBeDefined();
    });

    test('successfully validate volumes with tiering policy null', () => {
        expect(validateResponse(volumesNoTieringPolicy, volumesResponseSchema)).not.toBeDefined();
    });

    test('successfully validate fabric pool license status', () => {
        expect(validateResponse(licenseStatusValid, licenseStatusResponseSchema)).not.toBeDefined();
    });

    test('successfully validate fabric pool license status with no expiration value', () => {
        expect(validateResponse(licenseStatusNoExpiration, licenseStatusResponseSchema)).not.toBeDefined();
    });
});

describe('validate cloud-manager request schemas', () => {
    test('successfully validate discover cluster request', () => {
        expect(validateRequest(discoverRequest, discoverRequestSchema)).not.toBeDefined();
    });

    test('failed to validate discover cluster request with invalid cluster address', () => {
        function validate() {
            return validateRequest(discoverRequestInvalidAddress, discoverRequestSchema);
        }
        expect(validate).toThrow(/"clusterAddress" must be a valid ip address with a optional CIDR/);
    });
});
