const clusters = require('../../src/lib/cloud-manager/clusters');
const cloudManagerInfo = require('../../src/operations/cloud-manager-operations');
const multiTenancyOperations = require('../../src/operations/multi-tenancy-operation');
const { getAggregates, attachObjectStore } = require('../../src/lib/cloud-manager/aggregates');
const { getVolumes, modifyVolume } = require('../../src/lib/cloud-manager/volumes');
const {
    SOFT_RESTRICTION, HARD_RESTRICTION
} = require('../../src/utils/consts');

jest.mock('../../src/lib/cloud-manager/clusters');
jest.mock('../../src/operations/multi-tenancy-operation');
jest.mock('../../src/lib/cloud-manager/aggregates');
jest.mock('../../src/lib/cloud-manager/volumes');

function createCapacityObj(size = 1, unit = 'GB') {
    return {
        size,
        unit
    };
}

const clusterInfoNoTiering = {
    name: 'cluster1',
    publicId: 'cluster1',
    clusterUUID: 'cluster1',
    ontapClusterProperties: {
        ontapVersion: 'v1',
        clusterAddress: '1.1.1.1',
        nodeModel: 'node'
    },
    workingEnvironmentType: 'ON_PREM',
    status: {
        status: 'ON',
        failureCauses: null
    },
    capacityFeatures: {
        objectStoreConfig: null
    }
};

const clusterWithTiering = {
    name: 'cluster1',
    publicId: 'cluster1',
    clusterUUID: 'cluster1',
    ontapClusterProperties: {
        ontapVersion: 'v1',
        clusterAddress: '1.1.1.1',
        nodeModel: 'node'
    },
    workingEnvironmentType: 'ON_PREM',
    status: {
        status: 'SUCCESS',
        failureCauses: null
    },
    capacityFeatures: {
        objectStoreConfigExtended: {
            objectStoreConfig: {
                objectStoreName: 'object-store-name',
                s3Name: 'bucket',
                usedSpace: { size: 1, unit: 'TB' },
                providerType: 'AWS',
                server: 'server',
                tierLevel: 'Standard'
            }
        }
    }
};

const compositeAggregates = [
    {
        name: 'aggr1',
        state: 'online',
        objectStoreName: 'object-store-1',
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: true
    },
    {
        name: 'aggr2',
        state: 'online',
        objectStoreName: 'object-store-1',
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: true
    }
];

describe('setup tiering', () => {
    jest.setTimeout(40000);
    const VOLUME_IDENTIFIERS = [{
        name: 'vol1',
        svmName: 'svm_onprem'
    }, {
        name: 'vol2',
        svmName: 'svm_onprem'
    }];

    const VOLUMES = [{
        name: 'vol1',
        svmName: 'svm_onprem',
        aggregateName: 'aggr1',
        state: 'online'
    }, {
        name: 'vol2',
        svmName: 'svm_onprem',
        aggregateName: 'aggr1',
        state: 'online'
    }];

    const CLUSTER_INFO = {
        name: 'cluster1',
        publicId: 'cluster1',
        clusterUUID: 'cluster1',
        ontapClusterProperties: {
            ontapVersion: 'v1',
            clusterAddress: '1.1.1.1',
            nodeModel: 'node'
        },
        workingEnvironmentType: 'ON_PREM',
        status: {
            status: 'ON',
            failureCauses: null
        },
        capacityFeatures: {
            objectStoreConfigExtended: {
                objectStoreConfig: {
                    usedCapacity: {
                        size: 5,
                        unit: 'GB'
                    },
                    objectStoreName: 'objStore'
                },
                tierLevel: 'Standard'
            }
        }
    };

    const NON_COMPOSITE_AGGREGATE = {
        name: 'aggr1',
        state: 'online',
        objectStoreName: null,
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: true
    };

    beforeEach(() => {
        modifyVolume.mockClear();
        getVolumes.mockClear();
        attachObjectStore.mockClear();
        clusters.configureFabricPoolResources.mockClear();
    });

    test('succeed to setup tiering when cluster has already Fabric Pool resources', async () => {
        jest.setTimeout(40000);
        clusters.clusterInfo.mockReturnValue(CLUSTER_INFO);
        getAggregates.mockReturnValue(compositeAggregates);
        modifyVolume.mockReturnValue({});
        getVolumes.mockReturnValue(VOLUMES);
        expect(await cloudManagerInfo.setupTiering('AWS', 'aff1', 'eastus-1', 'aws', VOLUME_IDENTIFIERS)).toBeUndefined();
        expect(modifyVolume).toHaveBeenCalledTimes(2);
        expect(clusters.configureFabricPoolResources).not.toHaveBeenCalled();
    });

    test('succeed to setup tiering to azure when cluster has already Fabric Pool resources', async () => {
        jest.setTimeout(40000);
        clusters.clusterInfo.mockReturnValue(CLUSTER_INFO);
        getAggregates.mockReturnValue(compositeAggregates);
        modifyVolume.mockReturnValue({});
        getVolumes.mockReturnValue(VOLUMES);
        expect(await cloudManagerInfo.setupTiering('AZURE', 'aff1', 'eastus-1', 'aws', VOLUME_IDENTIFIERS)).toBeUndefined();
        expect(modifyVolume).toHaveBeenCalledTimes(2);
        expect(clusters.configureAzureFabricPoolResources).not.toHaveBeenCalled();
    });

    test('should succeed to setup tiering to aws when cluster doesnt have Fabric Pool resources yet', async () => {
        clusters.clusterInfo.mockReturnValueOnce(clusterInfoNoTiering).mockReturnValueOnce(clusterWithTiering);
        clusters.configureFabricPoolResources.mockReturnValue({ objectStoreConfigName: 'OSConfigName' });
        getAggregates.mockReturnValue(compositeAggregates);
        modifyVolume.mockReturnValue({});
        getVolumes.mockReturnValue(VOLUMES);
        expect(await cloudManagerInfo.setupTiering('AWS', 'aff1', 'eastus-1', 'aws', VOLUME_IDENTIFIERS)).toBeUndefined();
        expect(modifyVolume).toHaveBeenCalledTimes(2);
        expect(clusters.configureFabricPoolResources).toHaveBeenCalled();
    }, 60000);

    test('should succeed to setup tiering to azure when cluster doesnt have Fabric Pool resources yet', async () => {
        clusters.clusterInfo.mockReturnValueOnce(clusterInfoNoTiering).mockReturnValueOnce(clusterWithTiering);
        clusters.configureAzureFabricPoolResources.mockReturnValue({ objectStoreConfigName: 'OSConfigName' });
        getAggregates.mockReturnValue(compositeAggregates);
        modifyVolume.mockReturnValue({});
        getVolumes.mockReturnValue(VOLUMES);
        expect(await cloudManagerInfo.setupTiering('AZURE', 'aff1', 'eastus', undefined, VOLUME_IDENTIFIERS)).toBeUndefined();
        expect(modifyVolume).toHaveBeenCalledTimes(2);
        expect(clusters.configureFabricPoolResources).not.toHaveBeenCalled();
        expect(clusters.configureAzureFabricPoolResources).toHaveBeenCalledTimes(1);
    }, 60000);


    test('succeed to setup tiering when aggregates already tiered', async () => {
        clusters.clusterInfo.mockReturnValue(CLUSTER_INFO);
        getAggregates.mockReturnValue(compositeAggregates);
        modifyVolume.mockReturnValue({});
        getVolumes.mockReturnValue(VOLUMES);
        expect(await cloudManagerInfo.setupTiering('AWS', 'aff1', 'eastus-1', 'aws', VOLUME_IDENTIFIERS)).toBeUndefined();
        expect(modifyVolume).toHaveBeenCalledTimes(2);
    });

    test('succeed to setup tiering when aggregate is not composite', async () => {
        clusters.clusterInfo.mockReturnValue(CLUSTER_INFO);
        getAggregates.mockReturnValue([NON_COMPOSITE_AGGREGATE]);
        attachObjectStore.mockReturnValue({});
        modifyVolume.mockReturnValue({});
        getVolumes.mockReturnValue(VOLUMES);
        expect(await cloudManagerInfo.setupTiering('AWS', 'aff1', 'eastus-1', 'aws', VOLUME_IDENTIFIERS)).toBeUndefined();
        expect(modifyVolume).toHaveBeenCalledTimes(2);
        expect(attachObjectStore).toHaveBeenCalledTimes(2);
    });
});

describe('validateEligibleForTiering', () => {
    test('should return true', async () => {
        expect(cloudManagerInfo.validateEligibleForTiering(9.4, '1.1.1.1', 'aws')).toBeUndefined();
    });
    test('should throws when invalid ontap version AND AWS', async () => {
        function validateInvalidVersion() {
            cloudManagerInfo.validateEligibleForTiering(9.1, '1.1.1.1', 'aws');
        }
        expect(validateInvalidVersion).toThrow('Can\'t discover ONTAP Cluster. ONTAP Version 9.1 is not supported');
    });

    test('should throws when invalid ontap version AND AZURE', async () => {
        function validateInvalidVersion() {
            cloudManagerInfo.validateEligibleForTiering(9.3, '1.1.1.1', 'azure');
        }
        expect(validateInvalidVersion).toThrow('Can\'t discover ONTAP Cluster. ONTAP Version 9.3 is not supported');
    });
});

describe('checkTieringRestriction', () => {
    beforeEach(() => {
        attachObjectStore.mockClear();
    });

    test('no restrictions', async () => {
        attachObjectStore.mockReturnValue({});
        expect(await cloudManagerInfo.checkTieringRestriction('a2', 'aggr1', 'true')).toEqual(undefined);
    });

    test('soft restrictions', async () => {
        const errorMessage = 'Aggregate contains a thick volume.';
        attachObjectStore.mockImplementation(() => {
            throw new Error(errorMessage);
        });
        expect(await cloudManagerInfo.checkTieringRestriction('a2', 'aggr1', true)).toEqual({ restrictionType: SOFT_RESTRICTION, reason: `Soft restriction was detected: ${errorMessage} Fix this issue and try again` });
    });
    test('hard restrictions', async () => {
        const errorMessage = 'Hybrid aggregate was detected';
        attachObjectStore.mockImplementation(() => {
            throw new Error(errorMessage);
        });
        expect(await cloudManagerInfo.checkTieringRestriction('a2', 'aggr1', false)).toEqual({ restrictionType: HARD_RESTRICTION, reason: `Hard restriction was detected: ${errorMessage}` });
    });
});

describe('getSupportedVolumes', () => {
    const AGGREGATE = {
        name: 'aggr1',
        state: 'online',
        objectStoreName: 'object-store-1',
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: true
    };

    const VOLUME = {
        name: 'vol1',
        aggregateName: 'aggr1',
        svmName: 'svm_onprem',
        size: createCapacityObj(3),
        usedSize: createCapacityObj(2),
        state: 'online'
    };

    beforeEach(() => {
        getAggregates.mockClear();
        attachObjectStore.mockClear();
        getVolumes.mockClear();
    });

    test('no volumes returns - no volumes ', async () => {
        AGGREGATE.isObjectStoreAttachEligible = false;
        getAggregates.mockReturnValue([AGGREGATE]);
        getVolumes.mockReturnValue([]);
        expect(await cloudManagerInfo.getSupportedVolumes('a', 'clusterId')).toEqual([]);
    });

    test('no volumes returns - hard restriction', async () => {
        AGGREGATE.isObjectStoreAttachEligible = false;
        AGGREGATE.objectStoreName = undefined;
        getAggregates.mockReturnValue([AGGREGATE]);
        getVolumes.mockReturnValue([VOLUME]);
        const errorMessage = 'Hybrid aggregate was detected';
        attachObjectStore.mockImplementation(() => {
            throw new Error(errorMessage);
        });
        expect(await cloudManagerInfo.getSupportedVolumes('a', 'clusterId')).toEqual([]);
    });


    test('return volumes - soft restrictions restriction', async () => {
        AGGREGATE.isObjectStoreAttachEligible = true;
        getAggregates.mockReturnValue([AGGREGATE]);
        getVolumes.mockReturnValue([VOLUME]);
        const errorMessage = 'Aggregate contains a thick volume';
        attachObjectStore.mockImplementation(() => {
            throw new Error(errorMessage);
        });
        expect(await cloudManagerInfo.getSupportedVolumes('a', 'clusterId')).toHaveLength(1);
    });

    test('return volumes - no restrictions', async () => {
        AGGREGATE.isObjectStoreAttachEligible = true;
        getAggregates.mockReturnValue([AGGREGATE]);
        attachObjectStore.mockReturnValue({});
        expect(await cloudManagerInfo.getSupportedVolumes('a', 'clusterId')).toHaveLength(1);
    });

    test('return volumes - with soft restriction - state is offline', async () => {
        AGGREGATE.isObjectStoreAttachEligible = true;
        VOLUME.state = 'offline';
        getVolumes.mockReturnValue([VOLUME]);
        getAggregates.mockReturnValue([AGGREGATE]);
        const volumesWithRestriction = await cloudManagerInfo.getSupportedVolumes('a', 'clusterId');
        expect(volumesWithRestriction).toHaveLength(1);
        expect(volumesWithRestriction[0].tieringRestrictionReason).toBe('This volume’s state is offline. To enable tiering, a volume’s state must be \'online\'');
    });
});

describe('getSupportedVolumesNoReason', () => {
    const AGGREGATE = {
        name: 'aggr1',
        state: 'online',
        objectStoreName: 'object-store-1',
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: true
    };

    const VOLUME = {
        name: 'vol1',
        aggregateName: 'aggr1',
        svmName: 'svm_onprem',
        size: createCapacityObj(3),
        usedSize: createCapacityObj(2)
    };

    beforeEach(() => {
        getAggregates.mockClear();
        getVolumes.mockClear();
    });

    test('no volumes returns - no volumes ', async () => {
        getAggregates.mockReturnValue([AGGREGATE]);
        getVolumes.mockReturnValue([]);
        expect(await cloudManagerInfo.getSupportedVolumesNoReason('a', 'clusterId')).toEqual([]);
    });

    test('no volumes returns - hard restriction and no tiered volumes', async () => {
        AGGREGATE.isObjectStoreAttachEligible = false;
        AGGREGATE.objectStoreName = null;
        getAggregates.mockReturnValue([AGGREGATE]);
        getVolumes.mockReturnValue([VOLUME]);
        expect(await cloudManagerInfo.getSupportedVolumesNoReason('a', 'clusterId')).toEqual([]);
    });

    test('volumes returns - hard restriction  tiered volumes', async () => {
        AGGREGATE.isObjectStoreAttachEligible = false;
        AGGREGATE.objectStoreName = 'objectStoreName';
        getAggregates.mockReturnValue([AGGREGATE]);
        getVolumes.mockReturnValue([VOLUME]);
        expect(await cloudManagerInfo.getSupportedVolumesNoReason('a', 'clusterId')).toEqual([VOLUME]);
    });

    test('return volumes - no restrictions', async () => {
        AGGREGATE.isObjectStoreAttachEligible = true;
        getVolumes.mockReturnValue([VOLUME]);
        getAggregates.mockReturnValue([AGGREGATE]);
        expect(await cloudManagerInfo.getSupportedVolumesNoReason('a', 'clusterId')).toHaveLength(1);
    });
});

describe('getTieredVolumesSummary', () => {
    const AGGREGATE = {
        name: 'aggr1',
        state: 'online',
        objectStoreName: 'object-store-1',
        isInactiveDataReportingEnabled: true,
        isObjectStoreAttachEligible: true
    };

    const VOLUME = {
        name: 'vol1',
        aggregateName: 'aggr1',
        svmName: 'svm_onprem',
        size: createCapacityObj(3),
        usedSize: createCapacityObj(2),
        tieringPolicy: 'auto'
    };

    const VOLUME2 = {
        name: 'vol2',
        aggregateName: 'aggr1',
        svmName: 'svm_onprem',
        size: createCapacityObj(3),
        usedSize: createCapacityObj(2),
        tieringPolicy: 'none'
    };

    beforeEach(() => {
        getAggregates.mockClear();
        getVolumes.mockClear();
    });

    test('all volumes eligible', async () => {
        getVolumes.mockReturnValue([VOLUME, VOLUME]);
        getAggregates.mockReturnValue([AGGREGATE]);
        expect(await cloudManagerInfo.getTieredVolumesSummary('a', 'clusterId')).toEqual({
            eligibleVolumes: 2,
            tieredVolumes: 2
        });
    });

    test('Some volumes eligible', async () => {
        getVolumes.mockReturnValue([VOLUME, VOLUME2]);
        getAggregates.mockReturnValue([AGGREGATE]);
        expect(await cloudManagerInfo.getTieredVolumesSummary('a', 'clusterId')).toEqual({
            eligibleVolumes: 2,
            tieredVolumes: 1
        });
    });
});

describe('calculateSavingsOpportunitiesAndSavings', () => {
    test('cluster without tieredData total capacity and used capacity - savings should be 0 and saving opportunity capacity is 70% of non tiered volumes usedSize', async () => {
        const { savings, savingOpportunities: amount } = await
        cloudManagerInfo.calculateSavingsOpportunitiesAndSavings(7645041786, 0, 7516192768);
        expect(savings).toEqual(0);
        expect(amount).toEqual(33);
    });

    test('cluster with tierCapacity, usedCapacity and totalCapacity and coldCapacity', async () => {
        const { savings, savingOpportunities: amount } = await cloudManagerInfo.calculateSavingsOpportunitiesAndSavings(7645041786, 3006477107, 3006477107);
        expect(savings).toEqual(13);
        expect(amount).toEqual(13);
    });
});

describe('regionByS3Endpoint', () => {
    test('should get region by s3 endpoint when North Virginia', () => {
        expect(cloudManagerInfo.regionByS3Endpoint('s3.amazonaws.com')).toEqual('US East (N. Virginia)');
    });
    test('should get region by s3 endpoint', () => {
        expect(cloudManagerInfo.regionByS3Endpoint('s3-ap-northeast-1.amazonaws.com')).toEqual('Asia Pacific (Tokyo)');
    });
});

describe('updateColdDataReporting', () => {
    beforeEach(() => {
        clusters.enableColdDataReporting.mockClear();
    });
    test('should call enableDataReporting if supported ONTAP version', () => {
        cloudManagerInfo.updateColdDataReporting(9.4, 'clusterId', '10.10.10.10');
        expect(clusters.enableColdDataReporting).toHaveBeenCalled();
    });

    test('should skip enableDataReporting if non-supported ONTAP version', () => {
        cloudManagerInfo.updateColdDataReporting(9.3, 'clusterId', '10.10.10.10');
        expect(clusters.enableColdDataReporting).not.toHaveBeenCalled();
    });
});


describe('findTieringStats', () => {
    const clusterResponseNoTiering = {
        name: 'cluster1',
        publicId: 'cluster1',
        clusterUUID: 'cluster1',
        ontapClusterProperties: {
            ontapVersion: 'v1',
            clusterAddress: '1.1.1.1',
            nodeModel: 'node'
        },
        workingEnvironmentType: 'ON_PREM',
        status: {
            status: 'ON',
            failureCauses: null
        },
        capacityFeatures: {
            objectStoreConfig: null
        }
    };

    const clusterResponseFailed = {
        publicId: 'cluster1',
        status: {
            status: 'UNKNOWN',
            message: 'inactive agent'
        }
    };

    const clusterResponse = {
        name: 'cluster1',
        publicId: 'cluster1',
        clusterUUID: 'cluster1',
        ontapClusterProperties: {
            ontapVersion: 'v1',
            clusterAddress: '1.1.1.1',
            nodeModel: 'node'
        },
        workingEnvironmentType: 'ON_PREM',
        status: {
            status: 'ON',
            failureCauses: null
        },
        capacityFeatures: {
            objectStoreConfigExtended: {
                objectStoreConfig: {
                    objectStoreName: 'name',
                    usedSpace: {
                        size: 5,
                        unit: 'MB'
                    }

                },
                tierLevel: 'normal'
            }
        }
    };


    test('should get tieredCapacity 0- no clusters with tiering', async () => {
        multiTenancyOperations.getAllClustersInAccount.mockReturnValue([clusterResponseNoTiering]);
        expect(await cloudManagerInfo.findTieringStats('userToken', 'account')).toEqual({ tieredCapacity: 0, numberOfActiveClusters: 1, numberOfClusters: 1 });
    });


    test('should get tieredCapacity - 2 clusters with tiering', async () => {
        multiTenancyOperations.getAllClustersInAccount.mockReturnValue([clusterResponse, clusterResponse]);
        expect(await cloudManagerInfo.findTieringStats('userToken', 'account')).toEqual({ tieredCapacity: 10485760, numberOfActiveClusters: 2, numberOfClusters: 2 });
    });

    test('should get tieredCapacity 0 - no clusters at all', async () => {
        multiTenancyOperations.getAllClustersInAccount.mockReturnValue([]);
        expect(await cloudManagerInfo.findTieringStats('userToken', 'account')).toEqual({ tieredCapacity: 0, numberOfActiveClusters: 0, numberOfClusters: 0 });
    });

    test('should get tieredCapacity - 2 clusters, one with tiering', async () => {
        multiTenancyOperations.getAllClustersInAccount.mockReturnValue([clusterResponseNoTiering, clusterResponse]);
        expect(await cloudManagerInfo.findTieringStats('userToken', 'account')).toEqual({ tieredCapacity: 5242880, numberOfActiveClusters: 2, numberOfClusters: 2 });
    });

    test('should get tieredCapacity and message that indicates that information is not complete- 2 clusters, one with tiering one failed', async () => {
        multiTenancyOperations.getAllClustersInAccount.mockReturnValue([clusterResponseFailed, clusterResponse]);
        expect(await cloudManagerInfo.findTieringStats('userToken', 'account')).toEqual({ tieredCapacity: 5242880, numberOfActiveClusters: 1, numberOfClusters: 2 });
    });
});


describe('calculatePerformanceCapacity', () => {
    test('should calculate capacity if there is coldData and tieredData', () => {
        expect(cloudManagerInfo.calculatePerformanceCapacity(4398046511104, 2199023255552, 0))
            .toEqual({
                inactiveCapacity: 2199023255552,
                isEstimatedInactiveCapacity: false,
                activeCapacity: 2199023255552,
                usedPerformanceCapacity: 4398046511104
            });
    });


    test('should calculate capacity if there is no coldData and no tieredData', () => {
        expect(cloudManagerInfo.calculatePerformanceCapacity(5497558138880, 0, 0))
            .toEqual({
                inactiveCapacity: 3848290697216,
                isEstimatedInactiveCapacity: true,
                activeCapacity: 1649267441664,
                usedPerformanceCapacity: 5497558138880
            });
    });

    test('should calculate capacity if there is tieredData and no coldData', () => {
        expect(cloudManagerInfo.calculatePerformanceCapacity(4398046511104, 0, 0))
            .toEqual({
                inactiveCapacity: 3078632557773,
                isEstimatedInactiveCapacity: true,
                activeCapacity: 1319413953331,
                usedPerformanceCapacity: 4398046511104
            });
    });

    test('should calculate capacity if there is coldData and tieredData - volume', () => {
        expect(cloudManagerInfo.calculatePerformanceCapacity(4398046511104, 2199023255552, 0, true))
            .toEqual({
                inactiveCapacity: 2199023255552,
                isEstimatedInactiveCapacity: false,
                activeCapacity: 2199023255552,
                usedPerformanceCapacity: 4398046511104
            });
    });


    test('should calculate capacity if there is no coldData and no tieredData- volume', () => {
        expect(cloudManagerInfo.calculatePerformanceCapacity(5497558138880, 0, 0, true))
            .toEqual({
                inactiveCapacity: 3848290697216,
                isEstimatedInactiveCapacity: true,
                activeCapacity: 1649267441664,
                usedPerformanceCapacity: 5497558138880
            });
    });

    test('should calculate capacity if there is tieredData and no coldData - volume', () => {
        expect(cloudManagerInfo.calculatePerformanceCapacity(4398046511104, 0, 0, true))
            .toEqual({
                inactiveCapacity: 3078632557773,
                isEstimatedInactiveCapacity: true,
                activeCapacity: 1319413953331,
                usedPerformanceCapacity: 4398046511104
            });
    });
});
