const moment = require('moment');
const mongoose = require('mongoose');
const { DB_CONNECTION_STRING } = require('../../src/utils/consts');
const {
    createAccountAndClusterObject, getUsedCapacityOfAccount,
    getMeteringInfoForAccount, billableAccounts /* sendRecordsToAWS */
} = require('../../src/operations/marketplace-metering');
const clusters = require('../../src/lib/cloud-manager/clusters');
const multiTenancyOperations = require('../../src/operations/multi-tenancy-operation');
// const marketPlaceMetering = require('../../src/lib/aws/marketplace-metering');
const Account = require('../../src/models/account');
// const Metering = require('../../src/models/metering');
const getLogger = require('../../src/utils/logger/logger');

const logger = getLogger();

jest.mock('../../src/lib/cloud-manager/clusters');
jest.mock('../../src/lib/cloud-manager/multi-tenancy');
jest.mock('../../src/operations/multi-tenancy-operation');
// jest.mock('../../src/lib/aws/marketplace-metering');

const clusterResponse = {
    name: 'cluster1',
    publicId: 'cluster1',
    clusterUUID: 'cluster1',
    ontapClusterProperties: {
        ontapVersion: 'v1',
        clusterAddress: '1.1.1.1',
        nodeModel: 'node'
    },
    workingEnvironmentType: 'ON_PREM',
    status: {
        status: 'SUCCESS',
        failureCauses: null
    },
    capacityFeatures: {
        objectStoreConfig: {
            objectStoreName: 'name',
            usedSpace: {
                size: 5,
                unit: 'MB'
            },
            tierLevel: 'Standard'
        }
    }
};

const clusterResponse2 = {
    name: 'cluster2',
    publicId: 'cluster2',
    clusterUUID: 'cluster2',
    ontapClusterProperties: {
        ontapVersion: 'v1',
        clusterAddress: '2.2.2.2',
        nodeModel: 'node'
    },
    workingEnvironmentType: 'ON_PREM',
    status: {
        status: 'SUCCESS',
        failureCauses: null
    },
    capacityFeatures: {
        objectStoreConfig: {
            objectStoreName: 'name',
            usedSpace: {
                size: 5,
                unit: 'MB'
            },
            tierLevel: 'Standard'
        }
    }
};

const clusterResponseNoTiering = {
    name: 'cluster1',
    publicId: 'cluster1',
    clusterUUID: 'cluster1',
    ontapClusterProperties: {
        ontapVersion: 'v1',
        clusterAddress: '1.1.1.1',
        nodeModel: 'node'
    },
    workingEnvironmentType: 'ON_PREM',
    status: {
        status: 'SUCCESS',
        failureCauses: null
    },
    capacityFeatures: {
        objectStoreConfig: null
    }
};


clusters.clusterInfo.mockReturnValue(clusterResponse);

multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([clusterResponse, clusterResponse2]);

describe('marketplace metering', () => {
    describe('createAccountAndClusterObject', () => {
        test('createAccountAndClusterObject - should return {accountId, [clustersId]}', async () => {
            expect(await createAccountAndClusterObject('accountId1')).toEqual({
                accountId: 'accountId1',
                clusters: [clusterResponse, clusterResponse2]
            });
        });
    });

    describe('getUsedCapacityArray', () => {
        test('getUsedCapacityArray- find usedCapacity of all the clusters in a workspace', async () => {
            expect(await getUsedCapacityOfAccount('serviceToken', [clusterResponse, clusterResponse2])).toEqual(10485760);
        });
    });


    describe('getMeteringInfoForAccount', () => {
        test('getMeteringInfoForAccount- Happy path - 2 clusters in account', async () => {
            multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([clusterResponse, clusterResponse2]);
            expect(await getMeteringInfoForAccount('SERVICE_TOKEN', { accountId: '1', trialExpiry: Date.now(), awsSubscription: { customerIdentifier: 'customer', status: 'SUCCESS', updatedAt: Date.now() } }))
                .toEqual({ customerIdentifier: 'customer', usedCapacity: 10485760 });
        });


        test('getMeteringInfoForAccount- no clusters in account', async () => {
            multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([]);
            const result = await getMeteringInfoForAccount('SERVICE_TOKEN', { accountId: '1', trialExpiry: Date.now(), awsSubscription: { customerIdentifier: 'customer', status: 'SUCCESS', updatedAt: Date.now() } });
            expect(result)
                .toEqual({ customerIdentifier: 'customer', usedCapacity: 0 });
        });

        test('getMeteringInfoForAccount- no object-stores in the cluster', async () => {
            multiTenancyOperations.getAllClustersInAccountInternal.mockReturnValue([clusterResponseNoTiering]);
            const result = await getMeteringInfoForAccount('SERVICE_TOKEN', { accountId: '1', trialExpiry: Date.now(), awsSubscription: { customerIdentifier: 'customer', status: 'SUCCESS', updatedAt: Date.now() } });
            expect(result)
                .toEqual({ customerIdentifier: 'customer', usedCapacity: 0 });
        });
    });

    describe('billableAccounts', () => {
        beforeAll(async () => {
            try {
                await mongoose.connect(DB_CONNECTION_STRING, { useNewUrlParser: true, dbName: 'cloudTiering' });
            } catch (e) {
                logger.error('Cannot connect to mongoDB');
            }
        });

        afterAll(async done => {
            try {
                await mongoose.connection.close();
                done();
            } catch (e) {
                logger.error('Error closing connection', e);
            }
        });

        beforeEach(() => Account.deleteMany({}));

        test('billable users- no account to charge', async () => {
            const accountWithoutAWSSubscription = { name: 'test', accountId: '1', awsSubscription: { customerIdentifier: '2', status: 'unsubscribe-success' } };
            await Account.create(accountWithoutAWSSubscription);
            const result = await billableAccounts();
            expect(result).toHaveLength(0);
        });


        test('billable users- no account to charge unsubscribed user  and valid trial', async () => {
            const accountWithoutAWSSubscription = { name: 'test', accountId: '1', awsSubscription: { customerIdentifier: '2', status: 'unsubscribe-success' } };
            await Account.create(accountWithoutAWSSubscription);
            const result = await billableAccounts();
            expect(result).toHaveLength(0);
        });

        test('billable users- no account to charge subscribed user with valid trial', async () => {
            const accountWithoutAWSSubscription = { name: 'test', accountId: '1', awsSubscription: { customerIdentifier: '2', status: 'subscribe-success' } };
            await Account.create(accountWithoutAWSSubscription);
            const result = await billableAccounts();
            expect(result).toHaveLength(0);
        });

        test('billable users- 2 account2 in the DB one has subscribe-success the other has unsubscribe-success- one acocunt to charge', async () => {
            const date = moment().subtract(3, 'days');
            const accountWithoutAWSSubscription = { name: 'test', accountId: '1', awsSubscription: { customerIdentifier: '2', status: 'unsubscribe-success' } };
            const accountWithAWSSubscription = {
                name: 'test1', trialExpiry: date, accountId: '1', awsSubscription: { customerIdentifier: '2', status: 'subscribe-success' }
            };
            await Account.insertMany([accountWithoutAWSSubscription, accountWithAWSSubscription]);
            const result = await billableAccounts();
            expect(result).toHaveLength(1);
        });
    });

/*
    describe.only('sendRecordsToAWS', () => {
        beforeAll(async () => {
            try {
                await mongoose.connect(connectionString, { useNewUrlParser: true });
            } catch (e) {
                logger.error('Cannot connect to mongoDB');
            }
        });

        afterAll(async done => {
            try {
                await mongoose.connection.close();
                done();
            } catch (e) {
                logger.error('Error closing connection', e);
            }
        });

        beforeEach(() => Metering.deleteMany({}));

        // const generateMeteringId = () => Math.floor((1 + Math.random()) * 0x10000).toString(8);

        /!*
        function createProcessedAwsResponse(customerIdentifier, quantyity, timeStamp, status) {
            return {
                MeteringRecordId: generateMeteringId(),
                Status: status,
                UsageRecord: {
                    CustomerIdentifier: customerIdentifier,
                    Quantity: quantyity,
                    Timestamp: timeStamp
                }
            };
        }
*!/

        /!*        function createUnprocessedAwsResponse(customerIdentifier, quantyity, timeStamp) {
            return {
                CustomerIdentifier: customerIdentifier,
                Quantity: quantyity,
                Timestamp: timeStamp
            };
        } *!/

        test('sendRecordsToAWS- no unprocessedRecords- verify Metering collection has 3 records at the end', async () => {
            // let response;
            marketPlaceMetering.batchMeterUsage.mockReturnValue([{ Results: [], UnprocessedRecords: [] }]);
            /!*
            = jest.fn(records => {
                const processedRecords = records.map(record => {
                    const { customerIdentifier, quantity, timeStamp } = record;
                    if (customerIdentifier === 'CustomerNotSubscribed') {
                        return createProcessedAwsResponse(customerIdentifier, quantity, timeStamp, customerIdentifier);
                    } if (customerIdentifier === 'DuplicateRecord') {
                        return createProcessedAwsResponse(customerIdentifier, quantity, timeStamp, customerIdentifier);
                    }
                    return createProcessedAwsResponse(customerIdentifier, quantity, timeStamp, 'Success');
                });
                response = {
                    Results: processedRecords,
                    UnprocessedRecords: []
                };
                logger.info('RESPONSE insidde', response);
            }).mockReturnValue(response);
*!/

            const date = new Date().toISOString();
            const records = [createMeteringEntry('1', 2000, date),
                createMeteringEntry('DuplicateRecord', 3000, date),
                createMeteringEntry('CustomerNotSubscribed', 3000, date)];
            await sendRecordsToAWS(records, 1);
            expect(marketPlaceMetering.batchMeterUsage.mock.calls.length).toBe(1);
            // call to the sendToAWS function and then verify with 3 records verify metering has 3 records
            logger.info('@@@@@@@@@@@@@@@@@@@@', await marketPlaceMetering.batchMeterUsage([{
                CustomerIdentifier: 'customerIdentifier',
                Quantity: 20000,
                Timestamp: new Date().toISOString()
            }]));
        }); // all were saved as success in the metering collection
    });
*/
    // test('sendRecordsToAWS unprocessed Records that succeceded in the second time') //all were saved as success
    // test('sendRecordsToAWS - unprocessed records that are saved as failed')//failed in the secondtime
});
