const { URL } = require('url');
const uuidv4 = require('uuid/v4');
const nock = require('nock');
const { last, cloneDeep } = require('lodash');
const { validateSchema } = require('../../../src/utils/schema-validation');
const { agentSchema } = require('../../../src/joi-schemas/agents-schemas');
const {
    API_PATH_AGENTS, AGENTS_MGMT_ENDPOINT
} = require('../../../src/utils/consts');
const { escapeForwardSlash, findUserId } = require('../simulator-utils');
const { STATE, USERS } = require('../simulator-state');
const getLogger = require('../../../src/utils/logger/logger');

const logger = getLogger('simulator');

const agentsScope = nock(AGENTS_MGMT_ENDPOINT)
    .log(evt => logger.info(evt))
    // create agent
    .post(API_PATH_AGENTS, body => body.accountId)
    .delay(100)
    .reply(200, function (uri, requestBody) {
        const userId = findUserId(this.req.headers['x-user-token']);
        if (!USERS[userId]) {
            USERS[userId] = cloneDeep(STATE);
        }
        const { accountId, agent: placement } = JSON.parse(requestBody);
        const agentAndPlacement = Object.assign({}, { agentId: uuidv4() }, placement);
        validateSchema(agentAndPlacement, agentSchema);
        if (USERS[userId].AGENTS[accountId]) {
            USERS[userId].AGENTS[accountId].push(agentAndPlacement);
        } else {
            USERS[userId].AGENTS[accountId] = [agentAndPlacement];
        }
        return agentAndPlacement;
    })

    // get agents
    .get(API_PATH_AGENTS)
    .query(query => query.account_id)
    .delay(100)
    .reply(200, function (uri) {
        const requestUri = new URL(uri, AGENTS_MGMT_ENDPOINT);
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId]) {
            const accountId = requestUri.searchParams.get('account_id');
            const agentsResponse = { agents: USERS[userId].AGENTS[accountId] ? USERS[userId].AGENTS[accountId] : [] };
            return JSON.stringify(agentsResponse);
        }
        return [403, 'User is not authorized to perform this action'];
    })
    // get agent status
    .get(escapeForwardSlash(`${API_PATH_AGENTS}/.+`))
    .query(query => query.account_id)
    .delay(100)
    .reply(function (uri) {
        const userId = findUserId(this.req.headers['x-user-token']);
        const requestURL = new URL(uri, AGENTS_MGMT_ENDPOINT);
        const accountId = requestURL.searchParams.get('account_id');
        const agentId = last(requestURL.pathname.split('/'));
        if (USERS[userId]) {
            if (USERS[userId].AGENTS[accountId]) {
                const agentFromAgents = USERS[userId].AGENTS[accountId].find(agent => agent.agentId === agentId);
                if (agentFromAgents) {
                    return [200, { agent: agentFromAgents }];
                }
            }
            return [404, 'Agent not found'];
        }
        return [404, 'User was not found'];
    });

agentsScope.persist(true);
