const { URL } = require('url');
const Joi = require('joi');
const nock = require('nock');
const {
    isEqual, last, nth, set, cloneDeep
} = require('lodash');
const { validateSchema } = require('../../../src/utils/schema-validation');
const {
    clusterResponseSchema, discoverRequestSchema, configureFabricPoolResourcesRequest, licenseSchema,
    configureAzureFabricPoolResourcesRequest
} = require('../../../src/joi-schemas/cloud-manager-schamas');
const {
    CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_CLUSTERS_PATH_PREFIX, CLOUD_MANAGER_CLUSTER_METADATA_PATH_PREFIX,
    CLOUD_MANAGER_CONFIGURE_FP_RESOURCES_PATH_SUFFIX, CLOUD_MANAGER_FABRIC_POOL_LICENSE_PATH_SUFFIX,
    CLOUD_MANAGER_INACTIVE_DATA_REPORTING_PATH_SUFFIX, CLOUD_MANAGER_CONFIGURE_AZURE_FP_RESOURCES_PATH_SUFFIX
} = require('../../../src/utils/consts');
const { AZURE_LOCATIONS } = require('../../../src/lib/azure/locations');
const { escapeForwardSlash, findUserId } = require('../simulator-utils');
const { STATE, reset, USERS } = require('../simulator-state');
const getLogger = require('../../../src/utils/logger/logger');

const logger = getLogger('simulator');

const clustersScope = nock(CLOUD_MANAGER_ENDPOINT)
    .log(evt => logger.info(evt))
    // cluster metadata
    .get(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTER_METADATA_PATH_PREFIX}`)
    .delay(100)
    .reply(function () {
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const match = Object.values(USERS[userId].CLUSTERS).find(cluster => cluster.cluster.ontapClusterProperties && cluster.cluster.ontapClusterProperties.clusterAddress === this.req.headers.clusteraddress);
            if (match) {
                return [200, {
                    clusterName: match.cluster.name,
                    ontapVersion: match.cluster.ontapClusterProperties.ontapVersion,
                    nodeModels: match.cluster.ontapClusterProperties.nodeModels
                }];
            }
            return [404, {
                message: `Connection attempt to ${this.req.headers.clusteraddress}:443 failed`,
                causeMessage:
                        `SimplicatorBadRequestException: Connection attempt to ${this.req.headers.clusteraddress}:443 failed`
            }];
        }
        return [404, 'No account for user'];
    })

    // discover cluster
    .post(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, body => isEqual(Object.keys(body).sort(), Object.keys(Joi.describe(discoverRequestSchema).children).sort()))
    .delay(100)
    .reply(function (uri, requestBody) {
        const body = JSON.parse(requestBody);
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const cluster = Object.values(USERS[userId].CLUSTERS).find(x => x.cluster.ontapClusterProperties && x.cluster.ontapClusterProperties.clusterAddress === body.clusterAddress);
            if (cluster) {
                cluster.workspaceId = body.tenantId;
                cluster.agentId = this.req.headers['x-agent-id'];
                return [200, { publicId: cluster.cluster.publicId }];
            }
            return [404, {
                message: `Connection attempt to ${body.clusterAddress}:443 failed`,
                causeMessage:
                        `SimplicatorBadRequestException: Connection attempt to ${body.clusterAddress}:443 failed`
            }];
        }
        return [404, 'User not found'];
    })

    // create cluster
    .post(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`, body => {
        try {
            validateSchema(body, clusterResponseSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(200, function (uri, requestBody) {
        const cluster = JSON.parse(requestBody);
        const userId = findUserId(this.req.headers.authorization);
        if (!USERS[userId]) {
            USERS[userId] = cloneDeep(STATE);
        }

        USERS[userId].CLUSTERS[cluster.publicId] = {
            accountId: undefined,
            workspaceId: undefined,
            agentId: undefined,
            cluster
        };

        USERS[userId].VOLUMES[cluster.publicId] = [];
        USERS[userId].AGGREGATES[cluster.publicId] = [];

        return cluster;
    })

    // cluster info
    .get(escapeForwardSlash(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/.+`))
    .delay(100)
    .reply(function (uri) {
        const requestURL = new URL(uri, CLOUD_MANAGER_ENDPOINT);
        const publicId = last(requestURL.pathname.split('/'));
        const userId = findUserId(this.req.headers.authorization);
        if (!USERS[userId]) {
            // create new user in case there is no user- get cluster is another entry point to the system
            USERS[userId] = cloneDeep(STATE);
        }
        const cluster = Object.values(USERS[userId].CLUSTERS).find(x => x.cluster.publicId === publicId);
        if (publicId === 'eda35168-d036-4cce-9cc8-ca916ec36333') {
            return [504, {
                message: 'Agent is inactive',
                causeMessage:
                        'SimplicatorBadRequestException: agent is inactive'
            }];
        } if (cluster) {
            return [200, JSON.stringify(cluster.cluster)];
        }
        return [404, {
            message: `Cluster with id ${publicId} was not found`,
            causeMessage:
                        `SimplicatorBadRequestException: Cluster with id ${publicId} was not found`
        }];
    })

    // configure fabric pool resources
    .post(escapeForwardSlash(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/.+/${CLOUD_MANAGER_CONFIGURE_FP_RESOURCES_PATH_SUFFIX}`), body => {
        try {
            validateSchema(body, configureFabricPoolResourcesRequest);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(200, function (uri, requestBody) {
        const requestUri = new URL(uri, CLOUD_MANAGER_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 5);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = Object.values(USERS[userId].CLUSTERS).find(x => x.cluster.publicId === clusterId);
        const body = JSON.parse(requestBody);
        const { region, tierLevel } = body;
        if (cluster) {
            setTimeout(() => {
                set(USERS, `${userId}.CLUSTERS.${clusterId}.cluster.capacityFeatures.objectStoreConfigExtended`,
                    {
                        objectStoreConfig: {
                            objectStoreName: 'tiering_objectStore',
                            s3Name: 'bucket',
                            usedSpace: {
                                size: 0, unit: 'B'
                            },
                            providerType: 'aws_s3',
                            server: region === 'us-east-1' ? 's3.amazonaws.com' : `s3-${region}.amazonaws.com`
                        },
                        tierLevel
                    });
            }, 10000);
            return [200, { objectStoreName: 'tiering_objectStore' }];
        }
        return [404, {
            message: 'Cluster for fabric pool not found',
            causeMessage:
                'Cluster for fabric pool not found'
        }];
    })
    // configure azure fabric pool resources
    .post(escapeForwardSlash(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/.+/${CLOUD_MANAGER_CONFIGURE_AZURE_FP_RESOURCES_PATH_SUFFIX}`), body => {
        try {
            validateSchema(body, configureAzureFabricPoolResourcesRequest);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(200, function (uri, requestBody) {
        const requestUri = new URL(uri, CLOUD_MANAGER_ENDPOINT);
        const clusterId = nth(requestUri.pathname.split('/'), 5);
        const userId = findUserId(this.req.headers.authorization);
        const cluster = Object.values(USERS[userId].CLUSTERS).find(x => x.cluster.publicId === clusterId);
        const body = JSON.parse(requestBody);
        const { region } = body;
        if (cluster) {
            setTimeout(() => {
                set(USERS, `${userId}.CLUSTERS.${clusterId}.cluster.capacityFeatures.objectStoreConfigExtended`,
                    {
                        objectStoreConfig: {
                            objectStoreName: 'tiering_objectStore',
                            s3Name: 'container',
                            usedSpace: {
                                size: 0, unit: 'B'
                            },
                            providerType: 'AZURE',
                            server: AZURE_LOCATIONS[region] || region
                        },
                        tierLevel: 'hot'
                    });
            }, 10000);
            return [200, { objectStoreName: 'tiering_objectStore' }];
        }
        return [404, {
            message: 'Cluster for fabric pool not found',
            causeMessage:
                'Cluster for fabric pool not found'
        }];
    })


    // enableColdDataReporting
    .put(escapeForwardSlash(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/.+/${CLOUD_MANAGER_INACTIVE_DATA_REPORTING_PATH_SUFFIX}`), () => true)
    .delay(100)
    .reply(200, function (uri) {
        const requestURL = new URL(uri, CLOUD_MANAGER_ENDPOINT);
        const clusterId = nth(requestURL.pathname.split('/'), 5);
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            const cluster = Object.values(USERS[userId].CLUSTERS).filter(data => data.clusterId === clusterId);
            if (cluster) {
                return [200];
            }
            return [404, 'Cluster for enableColdDataReporting not found'];
        }
        return [404, 'User was not found'];
    })

    // delete/remove cluster
    .delete(escapeForwardSlash(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/.+`))
    .delay(100)
    .reply(200, function (uri) {
        const requestURL = new URL(uri, CLOUD_MANAGER_ENDPOINT);
        const publicId = last(requestURL.pathname.split('/'));
        const userId = findUserId(this.req.headers.authorization);
        if (USERS[userId]) {
            if (USERS[userId].CLUSTERS[publicId]) {
                delete USERS[userId].CLUSTERS[publicId];
                return [200];
            }
            return [404,
                {
                    message: `Cluster ${publicId} was not found`,
                    causeMessage:
                        `Cluster ${publicId} was not found`
                }];
        }
        return [404, 'USER was not found'];
    })

    // delete/remove all clusters
    .delete(escapeForwardSlash(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}`))
    .delay(100)
    .reply(200, function () {
        if (this.req.headers.authorization) {
            const userId = findUserId(this.req.headers.authorization);
            reset('CLUSTERS', userId);
            return [200];
        }
        reset('CLUSTERS');
        return [200];
    })
    .post(escapeForwardSlash(`${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_CLUSTERS_PATH_PREFIX}/.+/${CLOUD_MANAGER_FABRIC_POOL_LICENSE_PATH_SUFFIX}`), body => {
        try {
            validateSchema(body, licenseSchema);
            return true;
        } catch (err) {
            return false;
        }
    })
    .delay(100)
    .reply(200);

clustersScope.persist(true);
