const { URL } = require('url');
const uuidv4 = require('uuid/v4');
const nock = require('nock');
const {
    cloneDeep, last, nth
} = require('lodash');
const {
    MULTI_TENANCY_ENDPOINT, API_PATH_ACCOUNTS, API_PATH_RESOURCES, DISCOVER_LOCATION, API_PATH_RESOURCES_INTERNAL
} = require('../../../src/utils/consts');
const { escapeForwardSlash, findUserId } = require('../simulator-utils');
const getLogger = require('../../../src/utils/logger/logger');


const logger = getLogger('simulator');

const { STATE, USERS } = require('../simulator-state');

const multiTenancyScope = nock(MULTI_TENANCY_ENDPOINT)
    .log(evt => logger.info(evt))
    // get accounts
    .get(API_PATH_ACCOUNTS)
    .delay(100)
    .reply(200, function () {
        const userId = findUserId(this.req.headers['x-user-token']);
        if (!USERS[userId]) {
            USERS[userId] = cloneDeep(STATE);
        }
        const accounts = [];
        Object.keys(USERS[userId].ACCOUNTS).forEach(accountKey => accounts.push(USERS[userId].ACCOUNTS[accountKey]));
        return JSON.stringify(accounts);
    })

// create workspace
    .post(escapeForwardSlash(`${API_PATH_ACCOUNTS}/.+/.+`))
    .delay(100)
    .reply(function (uri) {
        const requestUri = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const accountId = nth(requestUri.pathname.split('/'), 4);
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId] && USERS[userId].ACCOUNTS[accountId]) {
            const workspaceName = nth(requestUri.pathname.split('/'), 5);
            const workspace = {
                workspacePublicId: uuidv4(),
                workspaceName
            };
            if (USERS[userId].WORKSPACES[accountId]) {
                USERS[userId].WORKSPACES[accountId].push(workspace);
            } else {
                USERS[userId].WORKSPACES[accountId] = [workspace];
            }
            return [200, workspace];
        }
        return [404, 'No account found'];
    })

    // create account
    .post(escapeForwardSlash(`${API_PATH_ACCOUNTS}/.+`))
    .delay(100)
    .reply(200, function (uri) {
        const userId = findUserId(this.req.headers['x-user-token']);
        if (!USERS[userId]) {
            USERS[userId] = cloneDeep(STATE);
        }
        const requestUri = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const account = {
            accountName: last(requestUri.pathname.split('/')),
            accountPublicId: uuidv4()
        };
        if (USERS[userId].ACCOUNTS) {
            USERS[userId].ACCOUNTS[account.accountPublicId] = account;
        } else {
            USERS[userId].ACCOUNTS = { [account.accountPublicId]: account };
        }
        return { accountPublicId: account.accountPublicId };
    })


    // get workspaces
    .get(escapeForwardSlash(`${API_PATH_ACCOUNTS}/.+/workspace`))
    .delay(100)
    .reply(200, function (uri) {
        const requestUri = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const accountId = nth(requestUri.pathname.split('/'), 4);
        const userId = findUserId(this.req.headers['x-user-token']);
        return JSON.stringify(USERS[userId].WORKSPACES[accountId] ? USERS[userId].WORKSPACES[accountId] : []);
    })

    // get resources
    .get(API_PATH_RESOURCES)
    .query(query => query['resource-type'] && query.account)
    .delay(100)
    .reply(200, function (uri) {
        const requestURL = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const tenantId = requestURL.searchParams.get('workspace');
        const accountId = requestURL.searchParams.get('account');
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId] && USERS[userId].WORKSPACES[accountId]) {
            if (tenantId) {
                if (USERS[userId].WORKSPACES[accountId].find(workspace => workspace.workspacePublicId === tenantId)) {
                    const clusters = Object.values(USERS[userId].CLUSTERS).filter(data => data.workspaceId === tenantId);
                    return JSON.stringify(clusters.map(cluster => ({
                        resourceIdentifier: cluster.cluster.publicId,
                        agentId: cluster.agentId,
                        resourceType: DISCOVER_LOCATION
                    })));
                }
                return [404];
            }
            const workspaces = USERS[userId].WORKSPACES[accountId].map(workspace => workspace.workspacePublicId);
            const clusters = Object.values(USERS[userId].CLUSTERS).filter(data => workspaces.includes(data.workspaceId));
            return JSON.stringify(clusters.map(cluster => ({
                resourceIdentifier: cluster.cluster.publicId,
                agentId: cluster.agentId,
                resourceType: DISCOVER_LOCATION
            })));
        }
        return [404];
    })
    // get resources
    .get(API_PATH_RESOURCES_INTERNAL)
    .query(query => query['resource-type'] && query.account)
    .delay(100)
    .reply(200, function (uri) {
        const requestURL = new URL(uri, MULTI_TENANCY_ENDPOINT);
        const tenantId = requestURL.searchParams.get('workspace');
        const accountId = requestURL.searchParams.get('account');
        const userId = findUserId(this.req.headers['x-user-token']);
        if (USERS[userId] && USERS[userId].WORKSPACES[accountId]) {
            if (tenantId) {
                if (USERS[userId].WORKSPACES[accountId].find(workspace => workspace.workspacePublicId === tenantId)) {
                    const clusters = Object.values(USERS[userId].CLUSTERS).filter(data => data.workspaceId === tenantId);
                    return JSON.stringify(clusters.map(cluster => ({
                        resourceIdentifier: cluster.cluster.publicId,
                        agentId: cluster.agentId,
                        resourceType: DISCOVER_LOCATION
                    })));
                }
                return [404];
            }
            const workspaces = USERS[userId].WORKSPACES[accountId].map(workspace => workspace.workspacePublicId);
            const clusters = Object.values(USERS[userId].CLUSTERS).filter(data => workspaces.includes(data.workspaceId));
            return JSON.stringify(clusters.map(cluster => ({
                resourceIdentifier: cluster.cluster.publicId,
                agentId: cluster.agentId,
                resourceType: DISCOVER_LOCATION
            })));
        }
        return [404];
    });


multiTenancyScope.persist(true);
