const axios = require('axios');
const uuidv4 = require('uuid/v4');
const { reset } = require('../simulator-state');
const {
    CLOUD_MANAGER_ENDPOINT, CLOUD_MANAGER_API_PREFIX, CLOUD_MANAGER_AGGREGATES_PATH_PREFIX, CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX
} = require('../../../src/utils/consts');
const { signJwtToken } = require('../simulator-utils');

require('../scopes/aggregates-scope');


const aggregate = {
    name: 'aggr1',
    state: 'up',
    objectStoreName: 'object-store-1',
    isInactiveDataReportingEnabled: false,
    isObjectStoreAttachEligible: true,
    usedCapacity: {
        size: 0,
        unit: 'TB'
    }
};

const aggregate2 = {
    name: 'aggr2',
    state: 'up',
    objectStoreName: null,
    isInactiveDataReportingEnabled: false,
    isObjectStoreAttachEligible: true,
    usedCapacity: {
        size: 0,
        unit: 'TB'
    }
};


describe('aggregate scope', () => {
    beforeEach(() => reset());

    test('create/get single aggregate', async () => {
        const clusterId = uuidv4();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${clusterId}`, aggregate, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const { data } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${clusterId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        expect(data).toEqual([aggregate]);
    });

    test('create/get single aggregate with different users', async () => {
        const clusterId = uuidv4();
        const clusterId2 = uuidv4();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${clusterId}`, aggregate, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const { data: aggregatesUser1 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${clusterId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${clusterId2}`, aggregate, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        const { data: aggregatesUser2 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${clusterId2}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        expect(aggregatesUser1).toEqual([aggregate]);
        expect(aggregatesUser2).toEqual([aggregate]);
    });

    test('create/get multiple aggregates', async () => {
        const clusterId = uuidv4();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${clusterId}`, aggregate, {
            headers: {
                authorization: signJwtToken()
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${clusterId}`, aggregate2, {
            headers: {
                authorization: signJwtToken()
            }
        });
        const { data } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${clusterId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        expect(data).toEqual([aggregate, aggregate2]);
    });

    test('create/get multiple aggregates with different users', async () => {
        const clusterId = uuidv4();
        const clusterId2 = uuidv4();

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${clusterId}`, aggregate, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const { data: aggreatesUser1 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${clusterId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken()
            }
        });


        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${clusterId2}`, aggregate, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${clusterId2}`, aggregate2, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });


        const { data: aggreatesUser2 } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${clusterId2}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken('user2')
            }
        });

        expect(aggreatesUser1).toEqual([aggregate]);
        expect(aggreatesUser2).toEqual([aggregate, aggregate2]);
    });


    test('attache object store - check only = true', async () => {
        const clusterId = uuidv4();

        const bodyRequest = {
            workingEnvironmentId: clusterId,
            objectStoreName: 'objectStore',
            aggregateName: 'aggr1',
            checkOnly: true
        };

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${clusterId}`, aggregate, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const response = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX}`, bodyRequest, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(response.status).toEqual(200);
    });

    test('attache object store to an aggregate of different user- should fail', async () => {
        try {
            const clusterId = uuidv4();

            const bodyRequest = {
                workingEnvironmentId: clusterId,
                objectStoreName: 'objectStore',
                aggregateName: 'aggr1',
                checkOnly: true
            };

            await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX}`, bodyRequest, {
                headers: {
                    authorization: signJwtToken('user2')
                }
            });
        } catch (err) {
            expect(err.response.status).toEqual(404);
        }
    });

    test('attache object store - check only = false, check an objectStoreName field exists', async () => {
        const clusterId = uuidv4();

        const bodyRequest = {
            workingEnvironmentId: clusterId,
            objectStoreName: 'objectStore',
            aggregateName: 'aggr2',
            checkOnly: false
        };

        await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}/${clusterId}`, aggregate2, {
            headers: {
                authorization: signJwtToken()
            }
        });

        const response = await axios.post(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATE_ATTACH_PATH_PREFIX}`, bodyRequest, {
            headers: {
                authorization: signJwtToken()
            }
        });
        expect(response.status).toEqual(200);

        const { data: aggreates } = await axios.get(`${CLOUD_MANAGER_ENDPOINT}${CLOUD_MANAGER_API_PREFIX}${CLOUD_MANAGER_AGGREGATES_PATH_PREFIX}?workingEnvironmentId=${clusterId}&fields=tieringProperties,usedCapacity`, {
            headers: {
                authorization: signJwtToken()
            }
        });

        expect(aggreates[0].objectStoreName).toEqual('objectStore');
    });
});
